/**
 * Title: ServletOne
 * Description:
 * Copyright:    Copyright (c) 2009
 * Company:      Morpheus
 * @author Stefano Adriani
 * @version 1.0
 */

package adriani.webapp ;

import adriani.webapp.util.* ;

import java.io.*;
import javax.servlet.*;
import javax.servlet.http.*;

/**
* A simple servlet mapped to the logic name "servletOne" in the <code>web.xml</code> configuration file.
* This class is instanciated by Tomcat only the first time the servlet is requested. The scope of this class,
* as the usage condition, are defined in the <code>web.xml</code> configuration file. For example:<br/><br/>
* <pre><code>
*     &lt;servlet-name&gt;servletOne&lt;/servlet-name&gt;
      &lt;url-pattern&gt;*.foo&lt;/url-pattern&gt;
* </code></pre>
* Specify that this servlet will be invoked when calling an URL ending with "foo". In the Test web app
* enviroment this should happen when calling the URL:<br/><br/>
*
* <code>http://localhost:8080/test/home/test.foo</code>
*/
public class ServletOne extends HttpServlet {

	/**
	* Method automatically invoked by the Servlet Container the first time the servlet is called.
	*/
	public void init(ServletConfig config) throws ServletException {
	
		super.init(config) ;
		if (Settings.DETAILS) System.out.println("*** " +  this.getClass().getName() + " initialized...") ;
	}

	/**
	* Method invoked when the servlet is activated by a normal URL request (e.g. an HTTP GET request). 
	* If it's necessary to read incoming HTTP parameters or cookies use the <code>request</code> object. 
	* To send back data to the client browser use the <code>response</code> object. 
	*
	* @param request the HTTP request (passed automatically by Tomcat).
	* @param response the HTTP response (returned automatically by Tomcat).
	*/
	public void doGet(HttpServletRequest request, HttpServletResponse response) 
		throws ServletException, IOException {
      
		PrintWriter out = response.getWriter() ;
		
		// Producing XHTML header
		out.write("<!DOCTYPE html PUBLIC '-//W3C//DTD XHTML 1.0 Transitional//EN' 'http://www.w3.org/TR/xhtml11-transitional.dtd'>\n") ;
		out.write("<html xmlns='http://www.w3.org/1999/xhtml' xml:lang='it' lang='it'>\n") ;
		out.write("<head>\n") ;
		out.write("<title>Web Page produced by ServletOne</title>\n") ;
		out.write("<link href='../css/style_01.css' rel='stylesheet' type='text/css'/>\n") ;
		out.write("<meta http-equiv='Content-Type' content='text/html; charset=iso-8859-1'/>\n") ;
		out.write("</head>\n") ;
		out.write("<body style='MARGIN:20px; BACKGROUND-COLOR: #BBEEFF'>\n") ;
		out.write("<p class='Main_Title' align='center'>SERVLET ONE PAGE</p>\n") ;
		out.write("<p class='Main_Font'><strong>Hello from the ServletOne class!</strong></p>\n") ;
		out.write("<p class='Main_Font'>" + getDescription(true).toString() +  "</p>\n") ;
		out.write("<br/>\n") ;
		
		// Producing XHTML BODY content
		ServletConfig servletConfig = getServletConfig() ;
		String listAction = servletConfig.getInitParameter("listAction") ;
		String saveAction = servletConfig.getInitParameter("saveAction") ;
		out.write("<p class='Main_Font'><strong>SERVLET CONTEXT PARAMETERS</strong></p>\n") ;
		out.write("<p class='Main_Font'><strong>listAction</strong>:&nbsp;" + listAction + "</p>\n") ;
		out.write("<p class='Main_Font'><strong>saveAction</strong>:&nbsp;" + saveAction + "</p>\n") ;

		// Producing XHTML footer
		out.write("<br/><hr/><br/>\n") ;
		out.write("<div align='center'><a class='Small_Link' href='javascript:history.back()' title='Go back to previous page'>Back</a></div>") ;
		out.write("</body>\n") ;
		out.write("</html>\n") ;
		
		// Printing class description of the STDOUT
		System.out.println() ;
		System.out.println( getDescription(false).toString() ) ;
	}
	
	/**
	* This method just return a <code>StringBuffer</code> object describing the purposes of this class. The
	* text message can be used for introspection or to print some documentation in the produced HTML page.
	*
	* @param html if <code>true</code> the text message is formatted for the HTML output.
	* @return a text message describing the purpose of this class.
	*/
	public StringBuffer getDescription(boolean html) {
	
		String eol ;
		if (html) eol = "<br/>" ; else eol = "\n" ;
		
		StringBuffer buffer = new StringBuffer() ;
		buffer.append("The ServletOne class it's a normal Java class having full name:" + eol + eol) ;
		buffer.append(this.getClass().getName() + eol + eol) ;
		buffer.append("The class offers the following method:" + eol + eol) ;
		buffer.append("public void doGet(HttpServletRequest request, HttpServletResponse response)" + eol + eol) ;
		buffer.append("This method receives the HTTP request, eventually reads some parameters from it, ") ;
		buffer.append("then produces the HTML response and send it back to the browser. ") ;
		buffer.append("If you are reading these words inside a Browser window, then you are looking at the ") ;
		buffer.append("HTML page produced by the ServletOne class (see the Java source for details)." + eol) ;
		buffer.append("Notice that this servlet is invoked any time the request URI ends with '.foo', as ") ;
		buffer.append("specified by the mapping rule written in the web.xml configuration file. Moreover, ") ;
		buffer.append("when this class is created Tomcat initializes it using the parameters defined in the ") ;
		buffer.append("web.xml file. In this case the parameters are: listAction and saveAction. ") ;
		buffer.append("Since this is a tutorial page, these parameters are not used. ") ;
		buffer.append("We can imagine that the actual purpose of these parameters could be to document and ") ;
		buffer.append("describe some API used to invoke commands to this servlet (e.g. LIST and SAVE)." + eol) ;
		return buffer ;
	}
  
} // END OF CLASS