/**
 * Title: SimpleHTTPClient
 * Description:
 * Copyright:    Copyright (c) 2009
 * Company:      Morpheus
 * @author Stefano Adriani
 * @version 1.0
 */

package adriani.webapp.util ;

import java.io.*;
import java.net.*;

/**
* Simple java client emulating a web browser. This class allows to invoke a web service
* using HTTP binding directly from the command line. For training reasons we use this class also in the
* <code>test_client.jsp</code> JSP page, but in that situation this client is not useful as when launched 
* from the command line (i.e. it's not so useful to emulate a browser inside a real browser).
*
*/
public class SimpleHTTPClient {

	/** CLASS ATTRIBUTES 																					*/
	public static boolean VERBOSE 	= true ;			
	
	/** DEFAULT VALUES	 																					*/	
	public String namespace 		= "http://services.webapp.adriani" ;
	public String start_Tag 		= "<q0:getValue><q0:param0>" ;
	public String end_Tag 			= "</q0:param0></q0:getValue>" ;

  	/** If <code>true</code> this client assume the SOAP request contains only one parameter.				*/
	protected boolean simple		= false ;
	
  	/** Contains the complete SOAP request for web services expecting multiple input parameters.			*/
	protected String soapRequest	= null ;

  	/** Beginning of each logging message																	*/
	protected String leadingName 	= "SimpleHTTPClient - " ;
	
	/** SOAP ACTION: NOT USED at the moment (see above)														*/
	public String action 			= "\"http://www.sirti.it/getCoverage/getCoverage\"" ;	
	
    /**
	* Main method used to call the SimpleHTTPClient from the command line.
    */
    public static void main(String[] args) {
	
		// 1. Reading input parameters
		String param = null ;
		if ((args != null) && (args.length > 0)) {
			param = new String(args[0]) ;
			if (SimpleHTTPClient.VERBOSE) System.out.println("\nSimpleSOAPClient - Launched with parameter: " + args[0] + "\n") ;
		} else {
			String value = "1231231231233" ;
			param = new String(value) ;
			if (SimpleHTTPClient.VERBOSE) System.out.println("\nSimpleSOAPClient - Launched using default: " + value) ;
		}
		
		// 2. Setting the client
		SimpleHTTPClient client = new SimpleHTTPClient() ;
		client.printRequest(param) ;
		
		// 3. Performing the HTTP call
		client.call("http://localhost:8080/axis2/services/simpleJava/getValue", param) ;
		
		// Eventually use following call to verify the HTTP request properties
		//client.call("http://localhost:8080/test/info/aaa_info.jsp?actionID=0", param) ;		
	}
	
// *************************				MUTATORS					***********************************	

	/**																									
	* Used to initialize this client when the SOAP request contains only one parameter. In this case we load
	* the opening and closing XML tags that will wrap around the unique parameter. By this mechanism the 
	* <code>call(String server, String param)</code> method will require only the parameter's value.<br/>
	* This method MUST NOT be used if the client must invoke a web service expecting multiple input parameters, 
	* in this case use the <code>init(String ns, String request)</code> method instead.
	*
	* @param ns the namespace used to define the XML elements in the service's WSDL.
	* @param tag_1 the opening tag of the input element.
	* @param tag_2 the closing tag of the output element.
	*/
	public void init(String ns, String tag_1, String tag_2) {
	
		this.simple = true ;	
		this.namespace = ns ;
		this.start_Tag = tag_1 ;
		this.end_Tag = tag_2 ;
	}
	
	/**
	* Call the web service.
	*
	* @param ns the namespace used to define the XML elements in the service's WSDL.
	* @param request the complete SOAP request, ready to be submitted to the web service.
	*/
	public void init(String ns, String request) {
	
		this.simple = false ;	
		this.namespace = ns ;
		this.soapRequest = request ;
	}

// *************************				SOAP CALL					***********************************
	
	/**
	* Call the web service.
	*
	* @param server the URL of the target server hosting the web service.
	* @param param the string value to be used for the main request element.
	*/
	public StringBuffer call(String server, String param) {
	
		StringBuffer response = null ;
		if (VERBOSE) System.out.println(leadingName + "Calling: " + server) ;
	
		try {
		
			// 1. Setting the URL connection
			URL u = new URL(server) ;
			URLConnection uc = u.openConnection() ;
			HttpURLConnection connection = (HttpURLConnection) uc ;
			connection.setDoOutput(true) ;
			connection.setDoInput(true) ;
			connection.setRequestMethod("POST") ;
			connection.setRequestProperty("SOAPaction", action) ;
			// 2. Setting the mandatory property (content-type)
			connection.setRequestProperty("content-type", "text/xml; charset=utf-8") ;			
			// Setting other not mandatory property (just to make sure)			
			connection.setRequestProperty("pragma", "no-cache") ;
			connection.setRequestProperty("connection", "close") ;
			connection.setRequestProperty("cache-control", "no-cache") ;			
			connection.setRequestProperty("accept", "application/soap+xml, application/dime, multipart/related, text/*") ;			
			
			// 3. Sending the SOAP request
			OutputStream out = connection.getOutputStream() ;
			Writer wout = new OutputStreamWriter(out) ;
			String message = null ;
			if (this.simple) message = getXML(param, false).toString() ;
				else message = this.soapRequest ;
			wout.write(message) ;
			wout.flush() ;
			wout.close() ;
			
			// 4. Getting the result
			response = getResponse(connection) ;
			System.out.println("SOAP RESPONSE:\n\n" + response + "\n") ;
			
		} catch (IOException e) {
			System.err.println(e);
		}
		
		return response ;
	}

// *************************			SOAP REQUEST CONSTRUCTION			********************************
	
	/**
	* Produce the XML string to be used in the SOAP request message.<br/>
	* <strong>Remark:</strong> use \" and not \' to wrap XML attributes, in order to allow the test_client.jsp
	* page to parse the input SOAP request using Javascript.
	*
	* @param param the string value to be used for the main request element.
	* @param text if true the XML string is produced in user-friendly format. Usa always false when
	* producing an actual XML SOAP message.
	*/
    public StringBuffer getXML(String param, boolean text) {

		String eol ;
		if (text) eol = "\r\n" ; else eol = "" ;
		
		StringBuffer message = new StringBuffer() ;
		message.append("<?xml version=\"1.0\"?>" + eol) ;
		message.append("<SOAP-ENV:Envelope ") ;
		// Defining XML namespaces
		message.append("xmlns:q0=\"" + namespace + "\" ") ;							// Defining my WSDL prefix
		message.append("xmlns:SOAP-ENV=") ;
		message.append("\"http://schemas.xmlsoap.org/soap/envelope/\" ") ;			// Defining SOAP-ENV prefix
		message.append("xmlns:xsd=") ;
		message.append("\"http://www.w3.org/2001/XMLSchema\" ") ; 					// Defining xsd prefix
		message.append("xmlns:xsi=") ;
		message.append("\"http://www.w3.org/2001/XMLSchema-instance\">" + eol) ; 	// Defining xsi prefix
		// Defining header
		message.append("<SOAP-ENV:Header>"  + eol);
		message.append("</SOAP-ENV:Header>" + eol);		
		// Defining body
		message.append("<SOAP-ENV:Body>" + eol);
		message.append(start_Tag + param + end_Tag + eol) ;
		message.append("</SOAP-ENV:Body>" + eol) ;
		message.append("</SOAP-ENV:Envelope>" + eol) ;
		return message ;
	}

// *************************			STANDARD OUTPUT MANAGEMENT			********************************

	/**
	* Print the SOAP request message on the STOUT (for testing and debug purposes). 
	*
	* @param the input parameter to be wrapped in the SOAP message.
	*/
    public void printRequest(String param) {
	
		System.out.println("\nSOAP REQUEST:") ;

		try {
			OutputStream out = System.out ;
			Writer wout = new OutputStreamWriter(out) ;
			String message = null ;
			if (this.simple) message = getXML(param, true).toString() ;			
				else message = this.soapRequest ;
			wout.write( "\n" + message + "\n") ;
			wout.flush();
		}
		catch (IOException e) {
			System.err.println(e) ;
		}
	}
	
	/**
	* Read the response from the stream bound to the <code>connection</code> and store it
	* into a <code>StringBuffer</code> object.
	*/
    public StringBuffer getResponse(HttpURLConnection connection) {
	
		StringBuffer message = new StringBuffer() ;		

		try {
			int c ;			
			int data = 0 ;
			InputStream in = connection.getInputStream() ;
			while ((c = in.read()) != -1) {
				data++ ;
				message.append((char) c) ;
			}
			in.close() ;
			if (VERBOSE) System.out.println(leadingName + "Retrieved " + data + " characters\n") ;
		} catch (IOException e) {
			System.err.println(e);
		}
		return message ;
	}	

} // END OF CLASS